var def = require('./imports').default;
var THREE = def.THREE;
var Edge = def.Edge;
var Node = def.Node;
var OrbitControls = def.OrbitControls;
var d3 = def.d3;
var ee = def.ee;
var Utils = require('./utils/utils');
var $ = require('jQuery');

module.exports = function (self) {
  self.addNode = function (node) {
    self.graph.addNode(node.id, node.data);
    self.graph.getNode(node.id).x = node.x;
    self.graph.getNode(node.id).y = node.y;
    self.graph.getNode(node.id).fx = node.fx;
    self.graph.getNode(node.id).fy = node.fy;
    self.graph.getNode(node.id).pinned = node.fx != undefined;
    Utils.prepNode(self.graph.getNode(node.id));
    self.drawNode(self.graph.getNode(node.id));
    self.graph.getNode(node.id).links = [];
    self.layoutInit = true;
  };

  self.removeNode = function (node) {
    if (!node || !node.id) {
      return
    }
    self.removeGraphEdges(node)

    self.undrawEdges(node);
    
    self.nodes.remove(node.renderData.draw_object);
    
    self.graph.removeNode(node.id);
    
    self.undrawNode(node);

    self.nodeCount -= 1;
  }

  self.removeGraphEdges = function(node) {
    var nodeLinks = self.graph.getLinks(node.id);
    if (nodeLinks) {
      nodeLinks.forEach(function(edge) {
        self.graph.removeLink(edge)
      });
    }
  }
  
  self.undrawNode = function(node) {
    var nodes = self.force.nodes();
    if (nodes) {
      for(var i = 0; i < nodes.length; i++) {
        if(nodes[i].id == node.id) {
          self.scene.remove(nodes[i].renderData.draw_object);
          nodes[i].renderData.textHolder.children[0].element.hidden = true;
          nodes.splice(i, 1);
        }
      }
    }
  }
  
  self.undrawEdges = function(node) {
    var positions = self.edges.attributes.position.array;
    var colors = self.edges.attributes.color.array;
    var z = 0;
    for (var i = 0; i < self.lineIndices.length + z; i++) {
      if (self.lineIndices[i - z].source.id == node.id
        || self.lineIndices[i - z].target.id == node.id) {
        for (var j = i - z; j < self.lineIndices.length; j++) {
          self.lineIndices[j] = self.lineIndices[j + 1];
          self.lineObjects[j * 2] = self.lineObjects[(j + 1) * 2];
          self.lineObjects[j * 2 + 1] = self.lineObjects[(j + 1) * 2 + 1];
          var k = j + 1
          if (k == self.lineIndices.length) {
            positions[j * 3] = 0;
            positions[j * 3 + 1] = 0;
            positions[j * 3 + 3] = 0;
            positions[j * 3 + 4] = 0;
            colors[j * 3] = 255;
            colors[j * 3 + 1] = 255;
            colors[j * 3 + 2] = 255;
            colors[j * 3 + 3] = 255;
            colors[j * 3 + 4] = 255;
            colors[j * 3 + 5] = 255;
          } else {
            positions[j * 3] = positions[k * 3];
            positions[j * 3 + 1] = positions[k * 3 + 1];
            positions[j * 3 + 3] = positions[k * 3 + 3];
            positions[j * 3 + 4] = positions[k * 3 + 4];
            colors[j * 3] = colors[k * 3];
            colors[j * 3 + 1] = colors[k * 3 + 1];
            colors[j * 3 + 2] = colors[k * 3 + 2];
            colors[j * 3 + 3] = colors[k * 3 + 3];
            colors[j * 3 + 4] = colors[k * 3 + 4];
            colors[j * 3 + 5] = colors[k * 3 + 5];
          }
        }
        self.lineIndices.pop();
        self.lineObjects.pop();
        self.lineObjects.pop();
        self.drawCount -= 2;
      }
    }
  }

  self.getNode = function (node_id) {
    return self.graph.getNode(node_id);
  };
  
  self.addEdge = function (source, target, visible=true) {
    self.graph.addLink(source.id, target.id);
    self.drawEdge(self.getNode(source.id), self.getNode(target.id), visible);
  };

  self.addNeighbors = function(node) {
    if (self.fakeNodes.length >= 3) {
      for (var i = 0; i < 5; i++) {
        self.removeNode(self.graph.getNode(self.fakeNodes[i].id));
      }
      self.fakeNodes = []
    }
    var id = self.lastNode.id;
    for (var i = 0; i < 5; i++) {
      id += 3;
      var newNode = $.extend(true, {}, self.lastNode);
      newNode.id = id;
      newNode.data.label = i;
      self.fakeNodes.push(newNode);
      self.addNode(newNode);
      self.addEdge(self.graph.getNode(node.id), self.graph.getNode(newNode.id));
    }
    self.lastNode = self.graph.getNode(id);
  }
}