var def = require('./imports').default;
var THREE = def.THREE;
var Edge = def.Edge;
var Node = def.Node;
var d3 = def.d3;
var ee = def.ee;

module.exports = function (self) {

  /**
   *  Change color of node edges
   */
  self.highlightNode = function(node, toggle) {
    if (toggle) {
      if (!def.NODE_NO_HIGHLIGHT) {
        node.renderData.draw_object.children[0].visible = true;
      } else {
        node.renderData.draw_object.material.color.setHex(node.renderData.hcolor);
      }
    } else {
      if (!def.NODE_NO_HIGHLIGHT) {
        self.lastHover.renderData.draw_object.children[0].visible = false;
      } else {
        self.lastHover.renderData.draw_object.material.color.set(new THREE.Color(self.lastHover.renderData.color));
      }
    }
  }

  /**
   *  Highlight edges from and to a node and hide others
   */
  self.highlightEdges = function(node, toggle) {
    const froms = [];
    const tos = [];
    for (var i = 0; i < self.selection.length; i++) {
      for (var j = 0; j < self.selection[i].links.length; j++) {
        froms.push(self.selection[i].links[j].fromId);
        tos.push(self.selection[i].links[j].toId);
      }
    }
    if (toggle) {
      for (var i = 0; i < node.links.length; i++) {
        froms.push(node.links[i].fromId);
        tos.push(node.links[i].toId);
      }
      self.graph.forEachNode((n) => {
        if (self.selection.indexOf(n) != -1
          || n == node) {
          self.colorNodeEdge(n, 0.5, 0.5, 1);
          for (var i = 0; i < n.linkObjs.length; i++) {
            n.linkObjs[i].linecolor = n.renderData.linecolor;
          }
        } else if (froms.indexOf(n.id) != -1
          || tos.indexOf(n.id) != -1) {
          self.colorNodeEdge(n, 0.1, 0.9, 1);
          if (n.renderData.draw_object.children[0]) {
            self.highlightNodeChildren(n, true);
          }
        } else {
          self.colorNodeEdge(n, 0.1, 0.9, 0.5);
          self.highlightNodeChildren(n, false);
        }
      });
    } else {
      self.graph.forEachNode((n) => {
        if (self.selection.indexOf(n) != -1
          || n == node) {
          self.colorNodeEdge(n, 0.5, 0.5, 1);
          for (var i = 0; i < n.linkObjs.length; i++) {
            n.linkObjs[i].linecolor = n.renderData.linecolor;
          }
        } else if (froms.indexOf(n.id) != -1
          || tos.indexOf(n.id) != -1) {
          self.colorNodeEdge(n, 0.1, 0.9, 1);
          if (n.renderData.draw_object.children[0]) {
            self.highlightNodeChildren(n, true);
          }
        } else {
          self.colorNodeEdge(n, 0.1, 0.9, 0.5);
          self.highlightNodeChildren(n, false);
        }
      });
    }
  }

  /**
   *  Change color of node edges
   */
  self.colorNodeEdge = function (node, dc, c, op) {
    node.renderData.draw_object.material.opacity = op;
    node.renderData.linecolor.r = self.darkMode ? dc : c;
    node.renderData.linecolor.g = self.darkMode ? dc : c;
    node.renderData.linecolor.b = self.darkMode ? dc : c;
  }

  /**
   *  Turn highlight for node on or off
   */
  self.highlightNodeChildren = function (node, h) {
    if (h && self.highlightNeighbors) {
      node.renderData.draw_object.children[0].material.color.set(def.ADJACENT_HIGHLIGHT);
      node.renderData.draw_object.children[0].visible = true;
    } else {
      node.renderData.draw_object.children[0].material.color.setHex(node.renderData.hcolor);
      node.renderData.draw_object.children[0].visible = false;
    }
  }
}