var def = require('./imports').default;
var THREE = def.THREE;
var Edge = def.Edge;
var Node = def.Node;
var OrbitControls = def.OrbitControls;
var d3 = def.d3;
var ee = def.ee;
var $ = require('jQuery');

module.exports = function (self) {
  
  var checked = true;

  self.onMouseMove = function (selection, mouseX, mouseY, button, ctrl) {
    
    if (self.dragOnMove) {
      self.updateSelection(mouseX, mouseY);
    }
    // mouse moves while no button is clicked
    if (!self.mouseDown) {
      self.onHover(selection);
      self.mouseStart = new THREE.Vector3(mouseX, mouseY, 0);

    // left button clicked
    } else if (button == 0) {

      // left-clicked empty space (i.e., not clicking a node)
      if (!self.dragging && self.selection.indexOf(selection) == -1 && !ctrl) {
        self.clearSelection();
      }

      // add nodes enclosed by selection box into node selection
      self.checkSelection(mouseX, mouseY);
      
      // control key not pressed 
      if (!ctrl) {

        // at least one node selected
        if (self.selection.length > 0) {

          // at this point: left button clicked, one or more nodes selected and dragged
          // reactivate (in D3's terminology: reheat) the force layout so that as user
          // drags nodes, other nodes will move along
          self.force.alpha(1);


          for (var i = 0; i < self.selection.length; i++) {

            self.selection[i].renderData.isSelected = false;

            if (!def.NODE_NO_HIGHLIGHT) {
              self.selection[i].renderData.draw_object.children[0].visible = false;
            } else {
              self.selection[i].renderData.draw_object.material.color.set(new THREE.Color(self.selection[i].renderData.color));
            }

            self.selection[i].renderData.textHolder.children[0].element.hideme = true;
          }
        }
      }

      if (self.selection.length > 0) {
        self.updateSelection(mouseX, mouseY);
      }

      checked = true;

      if (!self.dragging) {
        
        if (self.showBox) {
          self.selectBox.visible = true;
          self.showBox = false;
        }

        self.selectBox.position.x = mouseX;
        self.selectBox.position.y = mouseY;
        var diffx = self.mouseStart.x - mouseX;
        var diffy = self.mouseStart.y - mouseY;
        self.selectBox.scale.set(diffx, diffy, 1);
      } else {
        self.selectBox.visible = false;
      }
    }
  }

  self.onHover = function (node) {
    if (self.lastHover && self.selection.indexOf(self.lastHover) == -1) {
      self.highlightNode(self.lastHover, false);
      self.lastHover.renderData.textHolder.children[0].element.hideme = true;
      self.highlightEdges(node, false);
    }
    self.lastHover = node;
    if (node) {
      self.highlightNode(node, true);
      node.renderData.textHolder.children[0].element.hideme = false;
      self.highlightEdges(node, true)
    } else if (self.selection.length == 0) {
      self.graph.forEachNode((n) => {
        self.colorNodeEdge(n, 0.5, 0.5, 1);
        self.highlightNodeChildren(n, false);
      });
    }
  }

  self.onMouseDown = function (selection, mouseX, mouseY, button, ctrl) {
    self.mouseDown = true;
    if (button == 0) {      
      self.mouseStart = new THREE.Vector3(mouseX, mouseY, 0);
      if (!self.dragging) {
        self.showBox = true;
      }
      if (self.selection.indexOf(selection) == -1 && !ctrl) {
        for (var i = 0; i < self.selection.length; i++) {
          self.selection[i].renderData.isSelected = false;
          self.selection[i].renderData.isDragging = false;
          if (!def.NODE_NO_HIGHLIGHT) {
            self.selection[i].renderData.draw_object.children[0].visible = false;
          } else {
            self.selection[i].renderData.draw_object.material.color.set(new THREE.Color(self.selection[i].renderData.color));
          }
          self.selection[i].renderData.textHolder.children[0].element.hideme = true;
        }
        self.selection = [];
      }
      if (selection) {
        self.dragging = selection;
        if (!selection.renderData.isSelected) {
          if (self.selection.indexOf(selection) == -1) {
            self.selection.push(selection);
          }
          selection.renderData.isSelected = true;
          self.updateSelection(mouseX, mouseY);
        } else if (ctrl) {
          self.selection.splice(self.selection.indexOf(selection), 1);
          selection.renderData.isSelected = false;
          if (!def.NODE_NO_HIGHLIGHT) {
            selection.renderData.draw_object.children[0].visible = false;
          } else {
            selection.renderData.draw_object.material.color.set(new THREE.Color(self.selection[i].renderData.color));
          }
          selection.renderData.textHolder.children[0].element.hideme = true;
          self.dragging = null;
        }
      }
    }
  }

  self.onMouseUp = function (selection, mouseX, mouseY, button) {
    self.mouseDown = false;
    if (button == 0) {

      if (self.dragOnMove) {
        self.dragOnMove = false;
      }

      self.showBox = false;
      self.dragging = null;
      self.selectBox.visible = false;
      if(selection)
        selection.renderData.isDragging = false;
      
      self.ee.emit('select-nodes', self.selection);
    }
  }

  self.onRightClick = function (selection) {
    if (selection) {
      self.rightClickedNode = selection
    }
  }

  self.onRightClickCoords = function (event) {
    console.log('onRightClick called');
    self.ee.emit('right-click', {
      pageX: event.pageX,
      pageY: event.pageY,
    });
  }

}