var def = require('./imports').default;
var THREE = def.THREE;
var Edge = def.Edge;
var Node = def.Node;
var OrbitControls = def.OrbitControls;
var d3 = def.d3;
var ee = def.ee;
var Utils = require('./utils/utils');

module.exports = function (self) {
  /**
   *  Update the position and color of the edges
   */
  self.updateEdges = function () {
    self.edges.setDrawRange(0, self.drawCount);
    var total = self.drawCount;
    var positions = self.edges.attributes.position.array;
    var colors = self.edges.attributes.color.array;
    for (var i = 0; i < total; i += 2) {
      if (self.lineIndices[i/2 + 1]) {
        if (self.lineIndices[i/2 + 1].hide) {
          var v1pos = self.lineObjects[i].renderData.draw_object.position;
          var v2pos = self.lineObjects[i + 1].renderData.draw_object.position;
          positions[i * 3] = v1pos.x;
          positions[i * 3 + 1] = v1pos.y;
          positions[i * 3 + 3] = v2pos.x;
          positions[i * 3 + 4] = v2pos.y;
          colors[i * 3] = 255;
          colors[i * 3 + 1] = 255;
          colors[i * 3 + 2] = 255;
          colors[i * 3 + 3] = 255;
          colors[i * 3 + 4] = 255;
          colors[i * 3 + 5] = 255;
        } else {
          var v1pos = self.lineObjects[i].renderData.draw_object.position;
          var v2pos = self.lineObjects[i + 1].renderData.draw_object.position;
          var v1color = self.lineIndices[i/2].linecolor;
          var v2color = self.lineIndices[i/2].linecolor;
          positions[i * 3] = v1pos.x;
          positions[i * 3 + 1] = v1pos.y;
          positions[i * 3 + 3] = v2pos.x;
          positions[i * 3 + 4] = v2pos.y;
          colors[i * 3] = v1color.r;
          colors[i * 3 + 1] = v1color.g;
          colors[i * 3 + 2] = v1color.b;
          colors[i * 3 + 3] = v2color.r;
          colors[i * 3 + 4] = v2color.g;
          colors[i * 3 + 5] = v2color.b;
        }
      }
    }
    self.edges.attributes.position.needsUpdate = true;
    self.edges.attributes.color.needsUpdate = true;
  }

  /**
   *  Update the position of the labels
   */
  self.updateLabels = function () {
    var nodes = self.force.nodes();
    for (var i = 0; i < nodes.length; i++) {
      let thisNode = self.graph.getNode(nodes[i].id);
      if (thisNode.renderData.textHolder != undefined) {
        if (thisNode.renderData.textHolder.children[0].element.hideme != false
          && !thisNode.renderData.textHolder.children[0].element.override) {
          thisNode.renderData.textHolder.children[0].element.hidden = true;
        } else {
          thisNode.renderData.textHolder.children[0].element.hidden = false;
        }
      }
      thisNode.renderData.textHolder.position.x = thisNode.renderData.size + thisNode.renderData.draw_object.position.x;
      thisNode.renderData.textHolder.position.y = thisNode.renderData.draw_object.position.y;
    }
  }

  /**
   *  Update the position and color of the edges
   */
  self.updateNodes = function () {
    if (self.options.layout == "ngraph") {
      self.graph.forEachNode(function (node) {
        if (node.renderData) {
          node.renderData.draw_object.position.x = self.force.getNodePosition(node.id).x;
          node.renderData.draw_object.position.y = self.force.getNodePosition(node.id).y;
        }
      });
      if (!self.paused) {
        self.force.step();
      }
    } else if (self.options.layout == "d3") {
      var nodes = self.force.nodes();
      for (var i = 0; i < nodes.length; i++) {
        nodes[i].x = Math.max(-self.renderWidth, Math.min(self.renderWidth, nodes[i].x));
        nodes[i].y = Math.max(-self.renderHeight, Math.min(self.renderHeight, nodes[i].y));
        let thisNode = self.graph.getNode(nodes[i].id);
        if (thisNode && thisNode.renderData.draw_object) {
          if (!thisNode.pinned && !thisNode.renderData.isDragging) {
            thisNode.fx = null;
            thisNode.fy = null;
          } else {
            thisNode.fx = thisNode.x;
            thisNode.fy = thisNode.y;
          }
          thisNode.renderData.draw_object.position.x = nodes[i].x;
          thisNode.renderData.draw_object.position.y = nodes[i].y;
        }

      }
      if (!self.paused) {
        self.force.tick();
      }
    }
  }

  /**
   * Restrict camera translation
   */
  self.updateCamera = function () {
    self.controls.target.x -= self.controls.object.position.x
      - Math.max(-self.renderWidth, Math.min(self.renderWidth,
        self.controls.object.position.x));
    self.controls.object.position.x -= self.controls.object.position.x
      - Math.max(-self.renderWidth, Math.min(self.renderWidth,
        self.controls.object.position.x));
    self.controls.target.y -= self.controls.object.position.y
      - Math.max(-self.renderHeight, Math.min(self.renderHeight,
        self.controls.object.position.y));
    self.controls.object.position.y -= self.controls.object.position.y
      - Math.max(-self.renderHeight, Math.min(self.renderHeight,
        self.controls.object.position.y));
    self.setViewPortSize(self.ccamera);
  }
}